require 'test_helper'

module Vulgata
  class TranslationsControllerTest < ActionDispatch::IntegrationTest
    include Engine.routes.url_helpers

    setup do
      @post = create(:not_translated_post)
      @translation_state = translation_state_by_locale(@post, :de)
      @routes = Engine.routes
    end

    test "should get new" do
      login_as create(:sam_the_translator)
      get new_translations_path
      assert_response :success
    end

    test "should get pending translation state according to the user's locales" do
      sam_the_translator = create(:sam_the_translator)
      login_as sam_the_translator

      get new_translations_path
      assert_response :success

      translation_state = TranslationState.find_by(user: sam_the_translator, status: 2, locale: "zh-CN")
      assert_not_nil translation_state
    end

    test "should get edit because admin" do
      login_as create(:johnny_the_admin)
      get edit_translation_state_translations_path(translation_state_id: @translation_state)
      assert_response :success
    end

    test "should get edit because assigned to translation state" do
      german_title = "Mein erster Blogpost"
      german_body = "Ich liebe Hummus"
      translation_params = { title: german_title, body: german_body }

      bob_the_translator = create(:bob_the_translator)
      @translation_state.save_translation_version translation_params, bob_the_translator
      @translation_state.reload

      login_as bob_the_translator

      get edit_translation_state_translations_path(translation_state_id: @translation_state)
      assert_response :success
    end

    test "should fail on edit and redirect to root path because not admin" do
      sam_the_translator = create(:sam_the_translator)
      login_as sam_the_translator

      get edit_translation_state_translations_path(translation_state_id: @translation_state)
      assert_redirected_to root_path
    end

    test "should update translation because admin" do
      new_title = "Hallo"
      new_body = "Das ist die body ja"

      login_as create(:johnny_the_admin)

      patch translation_state_translations_path(translation_state_id: @translation_state.id, resource: { title: new_title, body: new_body })
      assert_redirected_to translation_state_path(id: @translation_state)

      @translation_state.reload
      assert_equal :done, @translation_state.status
    end

    test "should update translation because assigned to translation state" do

      bob_the_translator = create(:bob_the_translator)
      @translation_state.user = bob_the_translator
      @translation_state.status = :in_progress
      @translation_state.save

      login_as bob_the_translator

      new_title = "Mein erster Blogpost"
      new_body = "Ich liebe Hummus"

      patch translation_state_translations_path(translation_state_id: @translation_state.id, resource: { title: new_title, body: new_body })
      assert_redirected_to translation_state_path(id: @translation_state)

      @translation_state.reload
      assert_equal :done, @translation_state.status

    end

    test "should fail on update and redirect because not admin" do
      bob_the_translator = create(:bob_the_translator)
      login_as bob_the_translator

      new_title = "Mein erster Blogpost"
      new_body = "Ich liebe Hummus"

      patch translation_state_translations_path(translation_state_id: @translation_state.id, resource: { title: new_title, body: new_body })
      assert_redirected_to root_path
    end

    test "should stop translating" do
      bob_the_translator = create(:bob_the_translator)
      @translation_state.user = bob_the_translator
      @translation_state.status = :in_progress
      @translation_state.save

      login_as bob_the_translator

      patch translation_state_translations_path(translation_state_id: @translation_state.id, stop_translating: true)
      @translation_state.reload

      assert_equal :pending, @translation_state.status
      assert_nil @translation_state.user

      assert_redirected_to root_path
    end
  end
end
