require 'test_helper'

feature "Item Translation" do

  before do
    @post = create(:not_translated_post)
    @chinese_title = "我的第一篇文章"
    @chinese_body = "我喜欢Hummus"
  end

  scenario "Translator translates using the queue" do

    login_as create(:sam_the_translator)

    visit root_path
    click_link "Translate"

    fill_in "resource_title", with: @chinese_title
    fill_in "resource_body", with: @chinese_body

    click_button "Submit"

    assert_translation_saved_internally

  end

  scenario "Translator stops translating" do

    login_as create(:sam_the_translator)

    visit root_path
    click_link "Translate"

    click_button "Stop Translating"

     @post.reload
    saved_translation_state = translation_state_by_locale @post, :'zh-CN'

    # make sure translation state is saved correctly
    assert_equal :pending, saved_translation_state.status
    assert_equal nil, saved_translation_state.user

  end

  scenario "Admin translates through translation state" do

    login_as create(:johnny_the_admin)

    visit root_path
    click_link "Translations Queue"

    # should have as many pending translations as there are locales (except source locale)
    assert "Pending (3)", find(:css, ".nav-tabs li.active a").text

    chinese_translation_state = @post.translation_states.find { |ts| ts.locale == "zh-CN" }
    find(:css, "a[href='#{translation_state_path(chinese_translation_state.id)}']").click

    click_link "translate-btn"

    fill_in "resource_title", with: @chinese_title
    fill_in "resource_body", with: @chinese_body

    click_button "Submit"

    assert_translation_saved_internally

    assert page.has_content? @chinese_title
    assert page.has_content? @chinese_body

  end

  private

  def assert_translation_saved_internally
    @post.reload
    saved_translation_state = @post.translation_states.find { |ts| ts.locale == "zh-CN" }

    # make sure translation state is saved correctly
    assert_equal :done, saved_translation_state.status
    assert_equal User.current_user, saved_translation_state.user
    
    # translation should be saved locally
    saved_translation_data = saved_translation_state.translation_data
    assert_not_nil saved_translation_data
    assert_not_empty saved_translation_data

    assert_equal @chinese_title, saved_translation_data[:title]
    assert_equal @chinese_body, saved_translation_data[:body]

    # actual post shouldn't be modified yet
    assert_not_equal @chinese_title, @post.title_zh_cn
    assert_not_equal @chinese_body, @post.body_zh_cn
  end

end