require 'i18n'
require 'i18n/backend/vulgata'
require 'test_helper'

module Vulgata
  class I18nBackendTest < ActiveSupport::TestCase

    setup do
      I18n.backend = I18n::Backend::Vulgata.new
    end

    def test_yml_is_loaded
      I18n.backend.load_translations
      assert I18n.backend.translations.any?
      assert_equal "Hello world", I18n.backend.translate(:en, "hello") # vulgata_translations are lazy initialized
      assert I18n.backend.vulgata_translations.any?
    end

    def test_translation_states_created_for_all_locales
      assert_difference 'I18nTranslation.count', Vulgata::Helpers.available_locales.size do
        Vulgata::Helpers.available_locales.each do |locale|
          assert_difference 'TranslationState.count', Vulgata::Helpers.available_locales.size do
            I18n.backend.vulgata_translate(locale, "States should be created for new translations #{locale}")
          end
        end
      end
    end

    # def test_non_string_values_are_loaded_from_yml
    #   I18n.backend.load_translations
    # end

    def test_transaltion_is_updating_when_being_translated
      assert_difference 'I18nTranslation.count', 1 do
        assert_difference 'TranslationState.count', Vulgata::Helpers.available_locales.size do
          I18n.backend.vulgata_translate(:en, "Source Translation Example")
        end
      end

      translation_states = I18nTranslation.last.translation_states

      translation_states.each do |state|
        next if Vulgata::TranslationState.statuses[state.status] == Vulgata::TranslationState.statuses[:source]
        assert_difference 'I18nTranslation.count', 1 do
          state.save_translation_version(ActionController::Parameters.new({ value: "#{state.locale} translation" }), nil)
          state.approve_translation
        end
      end
    end

    # def test_count_option
    #   @backend
    # end

  end
end
